@@ -18,7 +18,7 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-/* This file was created with the aid of ``regdat.sh'' and ``/wip/cygport-git/gdb/gdb-7.10.1-1.i686/src/gdb-7.10.1/gdb/gdbserver/../regformats/i386/i386-mpx.dat''.  */
+/* This file was created with the aid of ``regdat.sh'' and ``/work/gdb-cygport/gdb-7.12.1-2.i686/src/gdb-7.12.1/gdb/gdbserver/../regformats/i386/i386-mpx.dat''.  */
 
 #include "server.h"
 #include "regdef.h"
@@ -66,20 +66,12 @@ static struct reg regs_i386_mpx[] = {
   { "xmm6", 2176, 128 },
   { "xmm7", 2304, 128 },
   { "mxcsr", 2432, 32 },
-  { "ymm0h", 2464, 128 },
-  { "ymm1h", 2592, 128 },
-  { "ymm2h", 2720, 128 },
-  { "ymm3h", 2848, 128 },
-  { "ymm4h", 2976, 128 },
-  { "ymm5h", 3104, 128 },
-  { "ymm6h", 3232, 128 },
-  { "ymm7h", 3360, 128 },
-  { "bnd0raw", 3488, 128 },
-  { "bnd1raw", 3616, 128 },
-  { "bnd2raw", 3744, 128 },
-  { "bnd3raw", 3872, 128 },
-  { "bndcfgu", 4000, 64 },
-  { "bndstatus", 4064, 64 },
+  { "bnd0raw", 2464, 128 },
+  { "bnd1raw", 2592, 128 },
+  { "bnd2raw", 2720, 128 },
+  { "bnd3raw", 2848, 128 },
+  { "bndcfgu", 2976, 64 },
+  { "bndstatus", 3040, 64 },
 };
 
 static const char *expedite_regs_i386_mpx[] = { "ebp", "esp", "eip", 0 };
